require 'json'
require 'net/http'
require 'uri'

class Cardano
	@@root_url = 'https://cardanoexplorer.com'

	##########
	# Blocks #
	##########

	# Returns the list of blocks, contained in pages
	def getPages()
		return JSON.parse(get('/api/blocks/pages'))['Right']
	end

	# Returns the list of total pages
	def getTotalPages()
		return JSON.parse(get('/api/blocks/pages/total'))['Right']
	end

	# Returns the summary of a given block
	# Takes a block hash as a string
	def getBlockSummary(block)
		return JSON.parse(get('/api/blocks/summary/' + block))['Right']
	end

	# Returns the summary of a given transaction
	# Takes a block hash as a string
	def getBlockTransactionSummary(txs)
		return JSON.parse(get('/api/blocks/txs/' + txs))['Right']
	end

	##########
	# Epochs #
	##########

	# Returns all paged slots in a specified epoch
	# Takes an epoch as a string
	def getEpochPages(epoch)
		return JSON.parse(get('/api/epochs/' + epoch))['Right']
	end

	# Returns information about a specified slot in an epoch
	# Takes an epoch and a slot as strings
	def getEpochSlot(epoch, slot)
		return JSON.parse(get('/api/epochs/' + epoch + '/' + slot))['Right']
	end

	###########
	# Genesis #
	###########

	# Returns address information of the genesis block
	def getGenesisAddress()
		return JSON.parse(get('/api/genesis/address'))['Right']
	end

	# Returns a summary of the genesis block
	def getGenesisSummary()
		return JSON.parse(get('/api/genesis/summary'))['Right']
	end

	# Returns the total pages of the genesis block
	def getGenesisPages()
		return JSON.parse(get('/api/genesis/address/pages/total'))['Right']
	end

	#########
	# Stats #
	#########

	# Returns transaction stats
	def getStats()
		return JSON.parse(get('/api/stats/txs'))['Right']
	end

	##########
	# Supply #
	##########

	# Returns the total ADA supply
	def getSupply()
		return JSON.parse(get('/api/supply/ada'))['Right']
	end

	################
	# Transactions #
	################

	# Returns information about the last N transactions
	def getLastTransactions()
		return JSON.parse(get('/api/txs/last'))['Right']
	end

	# Returns a summary of a specific transaction
	# Takes a transaction ID as a string
	def getTransactionSummary(txid)
		return JSON.parse(get('/api/txs/summary/' + txid))['Right']
	end

	####################
	# Helper functions #
	####################

	# get
	# Takes an endpoint string and a JSON body
	def get(endpoint)
		uri = URI(@@root_url + endpoint)
		
		# Send the request
		response = Net::HTTP.get(uri)

		return response
	end
end